<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */

/**
 * @file admin/key.php
 * @brief Ключ защиты определяет разрешённые (оплаченные) возможности видеорегистратора.
 */
require('../head.inc.php');
/**
 *
 * Функция возвращает mac-адрес ethernet устройства
 */
function getmacs()
{
    $mac = '<not ethernet card>';
    $v = popen($GLOBALS['GETMAC'], 'r');
    $mac = fread($v, 12);
    pclose($v);
    return $mac;
}

echo '<h1>' . $r_key . '</h1>' . "\n";

function print_eval_key()
{
    global $eval_key_info_text, $conf;
    $ip_cmd = sprintf('%s link show', $conf['ip']);
    echo "<div class='info'>$eval_key_info_text</div>\n";
    echo "<pre class='tty'>$ $ip_cmd\n";
    passthru($ip_cmd);
    echo "</pre>\n";
}

function mac_is_valid($mac)
{
    global $conf;

    system(
        sprintf(
            '%s link show | %s -iFq \'%s\'',
            $conf['ip'],
            $conf['grep'],
            $mac
        ),
        $retval
    );

    return !$retval;
}

function parse_v5_key_opts($keyOpts, $keyComments)
{
    global $strKeyOptions;

    if (!preg_match(
        '/^(([0-9A-Fa-f]{2}:?){4})$/',
        $keyOpts,
        $matches
    )) {
        return false;
    }

    $mac_opts = $matches[1];
    $limNr = sscanf(
        $mac_opts,
        "%x:%x:%x:%x",
        $net_videos,
        $v4l_videos,
        $net_audios,
        $analog_audios
    );

    if ($limNr !== 4) {
        // echo '<div class="error">' . $key_alien . '</div>' . "\n";
        return false;
    } else {
        // print
        echo '<div class="info">', $keyComments, PHP_EOL;
        if ($net_videos === 255) {
            if ($net_audios === 255) {
                if ($analog_audios === 255) {
                    $net_videos += $net_audios + $analog_audios + $v4l_videos;
                    $net_audios = $net_videos;
                    $v4l_videos = $analog_audios = 0;
                } else {
                    $net_videos += $net_audios + $analog_audios;
                    $net_audios = $net_videos;
                    $analog_audios = 0;
                }
            } else {
                $net_videos += $net_audios;
                $net_audios = $net_videos;
            }
        } else {
            $net_audios = $net_videos;
            $analog_audios = $v4l_videos;
        }
        echo sprintf($strKeyOptions, $net_videos, $v4l_videos, $net_audios, $analog_audios);
        echo '</div>' . "\n";
    }
    return true;
}

function parse_v6_key_opts($keyOpts, $keyComments)
{
    global $strKey6Options;

    if (!preg_match(
        '/^([0-9A-Fa-f]{4})$/',
        $keyOpts,
        $matches
    )) {
        return false;
    }

    $limNr = sscanf($matches[1], "%hx", $net_videos);
    $v4l_videos = $net_audios = $analog_audios = $net_videos;

    if ($limNr !== 1) {
        // echo '<div class="error">' . $key_alien . '</div>' . "\n";
        return false;
    } else {
        // print
        echo '<div class="info">', $keyComments, PHP_EOL;
        echo sprintf($strKey6Options, $net_videos);
        echo '</div>', PHP_EOL;
    }
    return true;
}


if (!isset($conf['key']) || empty($conf['key'])) {
    echo '<div class="error">key file is not defined, see "key" param in avreg.conf</div>' . "\n";
    print_eval_key();
    die();
}

if (0 === strpos($conf['key'], '/')) {
    $absKeyPath = &$conf['key'];
} else {
    $absKeyPath = $conf['conf-dir'] . '/' . $conf['key'];
}

if (!file_exists($absKeyPath)) {
    echo '<div class="error">' . sprintf($key_not_found, $conf['key']) . '</div>' . "\n";
    die();
}

if (!is_readable($absKeyPath)) {
    echo '<div class="error">' . sprintf($key_not_rd, $conf['key']) . '</div>' . "\n";
} else {
    echo '<h2>' . sprintf($r_key_state, $conf['key']) . '</h2>' . "\n";
    clearstatcache();
    $keyfile = fopen($absKeyPath, 'r');
    if (false === $keyfile) {
        die();
    }
    $keyComments = '';
    $keyVer = 5;
    while (!feof($keyfile)) {
        $line = trim(fgets($keyfile, 1024));
        if (preg_match('/^key_ver(\d+)_part(\d+)=(.*)$/i', $line, $matches)) {
            $keyVer = (int)$matches[1];
            $keyPart = (int)$matches[2];
            $keyPartBody = $matches[3];
            if ($keyPart !== 1) {
                continue;
            }

            if (strpos($keyPartBody, 'xx:xx:xx:xx:xx:xx') === 0) {
                print_eval_key();
                die();
            }

            $keyMac = substr($keyPartBody, 0, 17);
            if (!mac_is_valid($keyMac)) {
                // echo '<div class="error">' . $key_alien . '</div>' . "\n";
                echo '<div class="error">', $key_bad, '</div>', PHP_EOL;
                die();
            }

            $keyOpts = substr($keyPartBody, 18);
            if (empty($keyOpts)) {
                echo '<div class="error">', $key_bad, '</div>', PHP_EOL;
                die();
            }

            if (0 === stripos($keyOpts, 'xx')) {
                print_eval_key();
            } else {
                if (($keyVer === 5 && !parse_v5_key_opts($keyOpts, $keyComments)) ||
                    ($keyVer === 6 && !parse_v6_key_opts($keyOpts, $keyComments))
                ) {
                    echo '<div class="error">', $key_bad, '</div>', PHP_EOL;
                }
            }
            continue;
        } else {
            $keyComments .= htmlentities($line, ENT_QUOTES, $chset) . '<br>' . PHP_EOL;
        }
    }
    fclose($keyfile);
}

?>
<div>
    <a href="<?php echo $conf['prefix']; ?>/" target="_parent"><?php echo $strToMainPage; ?></a>
</div>
<?php
// phpinfo();
require('../foot.inc.php');
