<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */
/**
 * @file
 * @brief Статистика используемых ресурсов
 */
$USE_JQUERY=true;
require('../head.inc.php');

require('_vidserv_status.inc.php');

/**
 *
 * Функция определяющая критичность загрузки, возвращая цвет критичности
 * @param int $val текущее значение
 * @param int $warn_val предупреждающее значение
 * @param int $max_val максимальное критическое значение
 * @return string color
 */
function color_level($val, $warn_val, $max_val)
{
    if ($val >= $max_val) {
        return 'color:Red !important; font-weight: bold;';
    } else {
        if ($val >= $warn_val) {
            return 'color:#CC6600 !important; font-weight: bold;';
        } else {
            return 'color:#009900;';
        }
    }
}

echo '<h1>' . $r_stats . '</h1>' . "\n";

print_daemons_status($init_type, null);

$nproc = (int)exec('nproc --all');
$la_warn = (float) $nproc * 0.6;
$la_err = (float) $nproc * 0.8;

$la_a = preg_split('/\s+/', file_get_contents('/proc/loadavg'));
// tohtml(print_r($la_a, true));
$la1 = (float)($la_a[0]);
$la5 = (float)($la_a[1]);
$la15 = (float)($la_a[2]);

echo '<h2>' . sprintf($r_LoadAveragesHeaderFmt, $nproc) . '</h2>' . "\n";
print '<table cellspacing="0" border="1" cellpadding="5">' . "\n";
print '<tr bgcolor="' . $header_color . '">' . "\n";
print '<th>&nbsp;</th>' . "\n";
print '<th>1 min</th>' . "\n";
print '<th>5 min</th>' . "\n";
print '<th>15 min</th>' . "\n";
print '</tr>' . "\n";
print '<tr>' . "\n";
print '<th>Load averages</th>' . "\n";
print '<td align="center" style="' . color_level($la1, $la_warn, $la_err) . '">' .
    $la1 . '</td>' . "\n";
print '<td align="center" style="' . color_level($la5, $la_warn, $la_err) . '">' .
    $la5 . '</td>' . "\n";
print '<td align="center" style="' . color_level($la15, $la_warn, $la_err) . '">' .
    $la15 . '</td>' . "\n";
print '</tr>' . "\n";
print '</table><br />' . "\n";

echo '<h2>' . $r_cpu_stat . '</h2>' . "\n";
unset($lines);
exec($conf['iostat'] . ' -c 2>/dev/null', $lines, $retval);
if ($retval === 0) {
    $tmp = strtr(trim($lines[3]), array(',' => '.'));
    preg_match(
        '/^([0-9\.]+) +([0-9\.]+) +([0-9\.]+) +([0-9\.]+) +([0-9\.]+) +([0-9\.]+)/',
        $tmp,
        $matches
    );
    $allcpu_user = round((float)$matches[1]);
    $allcpu_nice = round((float)$matches[2]);
    $allcpu_system = round((float)$matches[3]);
    $allcpu_iowait = round((float)$matches[4]);
    $allcpu_steal = round((float)$matches[5]);
    $allcpu_idle = round((float)$matches[6]);
    $allcpu_total = $allcpu_user + $allcpu_system;
    print '<table cellspacing="0" border="1" cellpadding="5">' . "\n";
    print '<tr bgcolor="' . $header_color . '">' . "\n";
    print '<th>&nbsp;</th>' . "\n";
    print '<th>user</th>' . "\n";
    print '<th>system</th>' . "\n";
    print '<th>total</th>' . "\n";
    print '</tr>' . "\n";
    print '<tr>' . "\n";
    print '<th>Total avg. %CPU usage</th>' . "\n";
    print '<td align="center" style="' . color_level($allcpu_user, 40, 60) . '">' .
        $allcpu_user . '</td>' . "\n";
    print '<td align="center" style="' . color_level($allcpu_system, 10, 30) . '">' .
        $allcpu_system . '</td>' . "\n";
    print '<td align="center" style="' . color_level($allcpu_total, 50, 70) . '">' .
        $allcpu_total . '</td>' . "\n";
    print '</tr>' . "\n";
    print '</table><br />' . "\n";
}

/// команда определения использования диска
$cmd = 'top -bn1 -c -w120';
unset($outs);
exec('LANG=' . $locale_str . ' ' . $cmd, $outs, $retval);
print '<div class="info">' . "\n";
print '<legend class="tip-spoiler-title" title="More info">$ ' . $cmd . ' </legend>' . "\n";
echo '<pre class="tip-spoiler-body" style="display: none;">' . "\n";
$is_top_body = false;
$proc_nb = 0;
$last_non_avreg_proc = true;
foreach ($outs as &$line) {
    if (!$is_top_body) {
        if (preg_match('/^\s*\d+\s+/', $line)) {
            $is_top_body = true;
        }
    }

    if ($is_top_body) {
        if ($proc_nb > 5) {
            $is_avreg_proc = (!!preg_match('/(avregd|avreg\-mon|rtsp\-proxy)/', $line));
            if (!$is_avreg_proc) {
                if ($last_non_avreg_proc) {
                    echo "           ...\n";
                }
                $last_non_avreg_proc = false;
                continue;
            }
        }
        $proc_nb += 1;
    }

    echo $line . "\n";
}
echo '</pre>' . "\n";
print '</div>' . "\n";

echo '<h2>' . $r_mem_stat . '</h2>' . "\n";
/// Получение информации о памяти

$lines = file('/proc/meminfo');
$meminfo = array();

foreach ($lines as &$line) {
   if (preg_match ('/^([a-zA-Z\(\)_]+):\s+(\d+)\s+([a-zA-Z]+)/', $line, $matches)) {
      $par = $matches[1];
      $mod = $matches[2] === 'mB' ? 1024 : 1;
      $kB = (int)$matches[2] * $mod;

      $meminfo[$par] = $kB;
   }
}
unset($lines);

$cmd = 'free -h';
unset($outs);
exec('LANG=' . $locale_str . ' ' . $cmd, $outs, $retval);
print '<div class="info">' . "\n";
if ($meminfo['SwapTotal'] - $meminfo['SwapFree'] > $meminfo['SwapCached']) {
    echo '<pre style="color:Red;">';
} else {
    echo '<pre>';
}
echo '$ ' . $cmd . "\n\n";
foreach ($outs as $line) {
    echo $line . "\n";
}
echo '</pre>' . "\n";
print '</div>' . "\n";
unset($meminfo);

echo '<h2>' . $r_stats_df . '</h2>' . "\n";

$storage_dirs = get_storage_dirs();

print '<div class="info">' . "\n";
foreach ($storage_dirs as $profile_name => $profile_conf) {
    $storage_dir = $profile_conf['storage-dir'];
    /// Размер свободного места на диске в Гб
    $dfs = round(disk_free_space($storage_dir) / 1e9, 2);

    /// команда определения использования диска
    $cmd = $conf['df'] . ' -HT ' . $storage_dir;
    unset($outs);
    exec('LANG=' . $locale_str . ' ' . $cmd, $outs, $retval);
    if ($dfs < $conf['warn-disk-free']) {
        echo '<pre style="color:Red;">';
    } else {
        echo '<pre>';
    }
    echo $profile_name, ' $ ', basename($conf['df']), ' -HT ', $storage_dir, PHP_EOL, PHP_EOL;
    foreach ($outs as $line) {
        echo $line . "\n";
    }
    echo '</pre>' . "\n";
}
print '</div>' . "\n";
?>

<script type='text/javascript'>
    $(document).ready(function() {
        $('.tip-spoiler-body').hide();
        $('.tip-spoiler-title').click(function() {
            $(this).toggleClass('opened').toggleClass('closed').next().slideToggle();
        });
    });
</script>


<?php
require('../foot.inc.php');
