<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */
/**
 * @file pda/paginator.inc.php
 * @brief
 */
namespace Avreg;

/**
 * @class PdaPaginator
 * @brief Клас реализовывающий функцию пагинации
 *
 */
class PdaPaginator implements \Iterator
{
    private $ary;
    private $position;

    public function __construct($ary, $offset, $detail_uri_base, $conf, $limit = 10, $offset_name = 'off')
    {
        if (empty($ary) || !is_array($ary)) {
            throw new \InvalidArgumentException('empty input $ary');
        }

        if (empty($limit) || empty($detail_uri_base) || empty($conf)) {
            throw new \InvalidArgumentException('0 or empty $limit or $detail_uri_base or $conf');
        }

        $this->elems_all_nb = count($ary);
        $this->elems_per_page = (int)$limit;
        $this->offset_name = $offset_name;
        $this->cur_offset = $offset;
        $this->conf =& $conf;
        $this->uri_base = $detail_uri_base;

        $this->all_pages_nb = ceil($this->elems_all_nb / $this->elems_per_page);
        $this->cur_page = ($this->cur_offset / $this->elems_per_page) + 1;
        $this->cur_page_end = $this->cur_offset + $this->elems_per_page;

        $this->position = $this->cur_offset;
        if ($this->position > $this->elems_all_nb) {
            throw new \RuntimeException("offset ($this->position) > count(array) ($this->elems_all_nb)");
        }
        $this->ary =& $ary;
    }

    public function printAbove()
    {
        if ($this->cur_page <= 1) {
            return;
        }
        printf("<div style='text-align: center; width:%upx;'>\n", $this->conf['cellular']['thumb-image-width']);
        $scale = isset($_GET['scl']) ? $_GET['scl'] : 0;
        $FS_size = '';
        if (isset($_GET['aw']) && isset($_GET['ah'])) {
            $FS_size = sprintf('&aw=%s&ah=%s', $_GET['aw'], $_GET['ah']);
        }

        $uri = sprintf(
            '%s&%s=%u&scl=%u%s',
            $this->uri_base,
            $this->offset_name,
            ($this->elems_per_page * ($this->cur_page - 2)),
            $scale,
            $FS_size
        );

        print "<a href='$uri' title='пред.'>\n";
        printf(
            "<img src='%s/img/arrow_up_48x24.png' width=48 height=24 alt='пред.'></a></div>\n",
            $this->conf['prefix']
        );
    }

    public function printBelow()
    {
        $scale = isset($_GET['scl']) ? $_GET['scl'] : 0;
        $FS_size = '';
        if (isset($_GET['aw']) && isset($_GET['ah'])) {
            $FS_size = sprintf('&aw=%s&ah=%s', $_GET['aw'], $_GET['ah']);
        }

        if ($this->cur_page < $this->all_pages_nb) {
            printf("<div style='text-align: center; width:%upx;'>\n", $this->conf['cellular']['thumb-image-width']);
            $uri = sprintf(
                '%s&%s=%u&scl=%u%s',
                $this->uri_base,
                $this->offset_name,
                ($this->elems_per_page * $this->cur_page),
                $scale,
                $FS_size
            );

            print "<a href='$uri' title='след.'>\n";
            printf(
                "<img src='%s/img/arrow_down_48x24.png' width=48 height=24 alt='след.'></a></div>\n",
                $this->conf['prefix']
            );
        }

        if ($this->all_pages_nb > 1) {
            print "<div>Страницы: ";
            for ($p = 1; $p <= $this->all_pages_nb; $p++) {
                if ($p === $this->cur_page) {
                    print "<span style='font-size: 140%';>$p</span>&nbsp;";
                } else {
                    $uri = sprintf(
                        '%s&%s=%u&scl=%u%s',
                        $this->uri_base,
                        $this->offset_name,
                        ($this->elems_per_page * ($p - 1)),
                        $scale,
                        $FS_size
                    );
                    print "<a href='$uri' title='след.'>$p</a>&nbsp\n";
                }
            }
        }
        print "</div>\n";
    }

    public function rewind()
    {
        $this->position = $this->cur_offset;
    }

    public function current()
    {
        return $this->ary[$this->position];
    }

    public function key()
    {
        return $this->position;
    }

    public function next()
    {
        ++$this->position;
    }

    public function valid()
    {
        if ($this->position < $this->cur_page_end) {
            return isset($this->ary[$this->position]);
        } else {
            return false;
        }
    }
}
// class Paginator
