<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */

namespace Avreg;

require(__DIR__ . '/OnvifClient/OnvifAjaxController.php');

class OnvifClientController extends OnvifAjaxController
{
    public function getDeviceInfo($data = array())
    {
        $deviceInfo = $this->doRequest($data, \OnvifServices::DEVICEMANAGEMENT, 'GetDeviceInformation');

        if ($deviceInfo['isOk']) {
            $this->success(array(
                'DeviceInformation' => $deviceInfo['result']
            ));
        } else {
            $this->error('GetDeviceInformation failed: ' . $deviceInfo['error_message']);
        }
    }

    public function getProfiles($data = array())
    {
        $profiles = $this->doRequest($data, \OnvifServices::MEDIA, 'GetProfiles');

        if ($profiles['isOk']) {
            $this->success(array(
                'Profiles' => $profiles['result']
            ));
        } else {
            $this->error('GetProfiles() failed: ' . $profiles['error_message']);
        }
    }

    public function getProfile($data = array())
    {
        if (!isset($data['ProfileToken'])) {
            throw new \Exception('ProfileToken is not set');
        }

        $profile = $this->doRequest(
            $data,
            \OnvifServices::MEDIA,
            'GetProfile',
            array(
                'ProfileToken' => $data['ProfileToken']
            )
        );

        if ($profile['isOk']) {
            $this->success(array(
                'Profile' => $profile['result']
            ));
        } else {
            $this->error('GetProfile() failed: ' . $profile['error_message']);
        }
    }

    /**
     * @param array $data
     * @throws \Exception
     */
    public function getMediaUri($data = array())
    {
        if (!isset($data['ProfileToken'])) {
            throw new \Exception('ProfileToken is not set');
        }

        if (!isset($data['StreamType'])) {
            throw new \Exception('StreamType is not set');
        }

        if (!isset($data['TransportProtocol'])) {
            throw new \Exception('TransportProtocol is not set');
        }

        $streamUri = $this->doRequest(
            $data,
            \OnvifServices::MEDIA,
            'GetStreamUri',
            array(
                'StreamSetup' => array(
                    'Stream' => $data['StreamType'],
                    'Transport' => array(
                        'Protocol' => $data['TransportProtocol']
                    )
                ),
                'ProfileToken' => $data['ProfileToken']
            )
        );

        if ($streamUri['isOk']) {
            $this->success(array(
                'MediaUri' => $streamUri['result']->MediaUri
            ));
        } else {
            $this->error('GetStreamUri() failed: ' . $streamUri['error_message']);
        }
    }
}

$controller = new OnvifClientController();
