<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */

namespace Avreg;

require_once(__DIR__ . '/../../head-xhr.inc.php');
require_once(__DIR__ . '/../AjaxController.php');
require_once(__DIR__ . '/PtzInterface.inc.php');
require_once(__DIR__ . '/../get_cams_params.inc.php');

class AxisPtzController extends AjaxController implements PtzInterface
{
    public $camurl;

    public function __construct()
    {
        $this->setupCamera();

        parent::__construct();
    }

    protected function setupCamera()
    {
        global $adb;

        if (!isset($_POST['data']) || !isset($_POST['data']['cameraNumber'])) {
            throw new \Exception('Could not get camera params.');
        } else {
            $cameraNumber = $_POST['data']['cameraNumber'];
        }

        $cams_params = get_cams_params(
            'text_left, InetCam_IP, InetCam_http_port, InetCam_USER, InetCam_PASSWD, onvif_profile_token',
            $cameraNumber
        );

        $camData = $cams_params[$cameraNumber];

        $this->camurl = "http://{$camData['InetCam_USER']['v']}:{$camData['InetCam_PASSWD']['v']}@" .
            "{$camData['InetCam_IP']['v']}:{$camData['InetCam_http_port']['v']}";
    }

    public function getPtzSpaces($data = array())
    {
        $re = file_get_contents("$this->camurl/axis-cgi/com/ptz.cgi?query=limits");

        if (!empty($re)) {

            preg_match_all('/([^=\s]+)=([^=\s]+)/', $re, $r);
            $r = array_combine($r[1], array_map('floatval', $r[2]));

            $this->success(array(
                'coordSpaces' => array(
                    'zoom' => array(
                        'min' => $r['MinZoom'],
                        'max' => $r['MaxZoom']
                    ),
                    'pan' => array(
                        'min' => $r['MinPan'],
                        'max' => $r['MaxPan']
                    ),
                    'tilt' => array(
                        'min' => $r['MinTilt'],
                        'max' => $r['MaxTilt']
                    )
                )
            ));
        } else {
            $this->error();
        }
    }

    public function getPtzStatus($data = array())
    {
        $re = file_get_contents("$this->camurl/axis-cgi/com/ptz.cgi?query=position");

        if (!empty($re)) {

            preg_match_all('/([^=\s]+)=([^=\s]+)/', $re, $r);
            $r = array_combine($r[1], array_map('floatval', $r[2]));

            $this->success(array(
                'position' => array(
                    'pan' => $r['pan'],
                    'tilt' => $r['tilt'],
                    'zoom' => $r['zoom'],
                )
            ));
        } else {
            $this->error();
        }
    }

    public function moveAbsolute($data = array())
    {
        $re = file_get_contents(
            $this->camurl . "/axis-cgi/com/ptz.cgi?zoom="
            . $data['zoom'] . "&pan=" . $data['pan'] . "&tilt=" . $data['tilt']
        );

        if ($re !== false) {
            $this->success();
        } else {
            $this->error();
        }
    }

    public function moveStop($data = array())
    {
        $re = file_get_contents("$this->camurl/axis-cgi/com/ptz.cgi?move=stop");

        if ($re !== false) {
            $this->success();
        } else {
            $this->error();
        }
    }

    public function getPtzPresets($data = array())
    {
        $this->error('Not implemented', 501);
    }

    public function setPreset($data = array())
    {
        $this->error('Not implemented', 501);
    }

    public function removePreset($data = array())
    {
        $this->error('Not implemented', 501);
    }

    public function gotoPreset($data = array())
    {
        $this->error('Not implemented', 501);
    }

    public function gotoHomePosition($data = array())
    {
        $re = file_get_contents("$this->camurl/axis-cgi/com/ptz.cgi?move=home");

        if ($re !== false) {
            $this->success();
        } else {
            $this->error();
        }
    }

    public function setHomePosition($data = array())
    {
        $re = file_get_contents("$this->camurl/axis-cgi/com/ptzconfig.cgi?setserverpresetname=home&home=yes");

        if ($re !== false) {
            $this->success();
        } else {
            $this->error();
        }
    }
}

$controller = new AxisPtzController();

/* vim: set expandtab smartindent tabstop=4 shiftwidth=4: */
