<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */

define('CFM_MODE_R', 0x4);
define('CFM_MODE_W', 0x2);
define('CFM_MODE_X', 0x1);

/***
 * Проверяет права доступа к файлу
 *
 * При отсутствии доступа к файлу генерирует исключение ErrorException,
 * а при неправильном параметре InvalidArgumentException.
 *
 * @param $file_path - путь к файлу
 * @param $max_uid - максимально допустимый uid, если отрицательное или false - любой
 * @param $max_gid - максимально допустимый gid, если отрицательное или false - любой
 * @param $user_mode_mask,$gr_mode_mask,$other_mode_mask - маски режима доступа для пользователя, группы и остальных, соотв.
 *
 * @return false|true
 */
function check_file_mode ($file_path, $max_uid, $max_gid, $user_mode_mask, $gr_mode_mask, $other_mode_mask)
{
   $stat_ar = @stat($file_path);

   if (false === $stat_ar) {
      $e = error_get_last();
      throw new ErrorException($e['message'], 0, $e['type'],  $e['file'], $e['line']);
   }

   // var_dump($stat_ar);

   // check user/group
   if ($max_uid !== false) {
      if (!is_numeric($max_uid) || false === settype($max_uid, 'integer')) {
         throw new InvalidArgumentException("unexpected \$max_uid value: \"$max_uid\"");
      }
      if ($max_uid >= 0 && $stat_ar['uid'] > intval($max_uid)) {
         return false;
      }
   }
   if ($max_gid !== false) {
      if (!is_numeric($max_gid) || false === settype($max_gid, 'integer')) {
         throw new InvalidArgumentException("unexpected \$max_gid value: \"$max_gid\"");
      }
      if ($max_gid >= 0 && $stat_ar['gid'] > intval($max_gid)) {
         return false;
      }
   }

   // check mode
   if (!is_numeric($user_mode_mask) || false === settype($user_mode_mask, 'integer')) {
      throw new InvalidArgumentException("unexpected \$user_mode_mask value: \"$user_mode_mask\"");
   }
   $u_mask = intval($user_mode_mask);

   if (!is_numeric($gr_mode_mask) || false === settype($gr_mode_mask, 'integer')) {
      throw new InvalidArgumentException("unexpected \$gr_mode_mask value: \"$gr_mode_mask\"");
   }
   $g_mask = intval($gr_mode_mask);

   if (!is_numeric($other_mode_mask) || false === settype($other_mode_mask, 'integer')) {
      throw new InvalidArgumentException("unexpected \$other_mode_mask value: \"$other_mode_mask\"");
   }
   $o_mask = intval($other_mode_mask);

   $mode_s = substr(decoct($stat_ar['mode']), -3);

   // var_dump($mode_s);

   $u = intval(substr($mode_s, 0, 1));
   $g = intval(substr($mode_s, 1, 1));
   $o = intval(substr($mode_s, 2, 1));

   // echo "file mode: $u$g$o, mask: $u_mask$g_mask$o_mask\n";

   if ( ($u | $u_mask) != $u_mask) {
      return false;
   } 
   if ( ($g | $g_mask) != $g_mask) {
      return false;
   } 
   if ( ($o | $o_mask) != $o_mask) {
      return false;
   } 

   return true;
}
