<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */

require_once(__DIR__ . '/check-file-mode.inc.php');

define('MAX_ARRAY_PARAM_DEPTH', 10);

// return null or variable of needing type
function __settype($value_def, $value_conf)
{
    $t = gettype($value_def);

    switch ($t) {
        case 'boolean':
            $ret = filter_var($value_conf, FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE);
            break;
        case 'integer':
            $ret = filter_var($value_conf, FILTER_VALIDATE_INT, FILTER_NULL_ON_FAILURE);
            break;
        case 'double':
            $ret = filter_var($value_conf, FILTER_VALIDATE_FLOAT, FILTER_NULL_ON_FAILURE);
            break;
        case 'NULL':
            if (settype($value_conf, 'string')) {
                $ret = $value_conf;
            } else {
                $ret = null;
            }
            break;
        case 'string':
            $ret = $value_conf;
            break;
        case 'array':
              $ret = preg_split("/\s*[,;]\s*+/", $value_conf);
            break;
        default:
            $ret = null;
    }
    return $ret;
} /* __settype() */


/***
 * Функция извлекает конфигурационные параметры из файла
 * @param array $_conf масив текущих параметров
 * @param string $sections секция или список секций или массив секций в которой искать параметры,
 *                         если не задана - то только глобальная
 * @param string $path путь к файлу
 * @param array $params параметры, которые нужно извлечь, null если все
 * @return array масив параметров или вызывает исключение при ошибке
 */
function confparse($_conf, $sections = false, $path = '/etc/avreg/avreg.conf', $params = null)
{
    global $SERVER_SOFT;

    $file_ext = pathinfo($path, PATHINFO_EXTENSION);

    try {
        if (!strcasecmp($file_ext, 'secret')) {
            $cfm = check_file_mode($path, 0, 999, 6,4,0);
            if (!$cfm) {
                throw new InvalidArgumentException ("secret conf file \"$path\" should have a \"root:www-data 0640\" perms");
            }
        } else {
            $cfm = check_file_mode($path, 0, 999, 6,4,4);
            if (!$cfm) {
                throw new InvalidArgumentException ("conf file \"$path\" should have a \"root:root(or avreg) 0644\" perms");
            }
        }
        if ($cfm) {
            clearstatcache(); // to next request
        }
    } catch (ErrorException $e) {
        // stat() failed
        clearstatcache(); // non need if ENOENT but ...
        throw($e);
    }

    $confile = fopen($path, 'r');
    if (false === $confile) {
        $e = error_get_last();
        $errno = 0;
        if ($e['type'] <= 2) {
            if ( false !== strpos($e['message'], 'denied')) {
                $errno = 13; // EACCES
            } else if (false !== strpos($e['message'], 'found')) {
                $errno = 2;  // ENOENT
            }
        }
        throw new ErrorException($e['message'], $errno, $e['type'],  $e['file'], $e['line']);
    }

    if (!empty($sections)) {
        switch (gettype($sections)) {
            case 'string':
                $app_sections_array = explode(',', $sections);
                $use_app_sections = (count($app_sections_array) > 0);
                break;
            case 'array':
                $app_sections_array = &$sections;
                $use_app_sections = (count($app_sections_array) > 0);
                break;
            default:
                $use_app_sections = false;
        }

        if ($use_app_sections && $SERVER_SOFT['name_lo']) {
            array_push($app_sections_array, 'if-' . $SERVER_SOFT['name_lo']);
        }
    }

    $skip_section = false;
    $linenr = 0;
    $ret_array = array();
    $res = true;

    while (!feof($confile)) {
        $line = trim(fgets($confile, 1024));
        $linenr++;
        if (empty($line)) {
            continue;
        }

        if (preg_match('/^\s*[;#]/', $line)) {
            continue;
        } /* skip comments */

        if (preg_match('/^([^\s=]+)[\s=]*\{$/', $line, $matches)) {
            # begin section
            $secName = $matches[1];
            if (!$use_app_sections || !in_array($secName, $app_sections_array)) {
                $skip_section = true;
            }
            continue;
        }

        if (preg_match('/.*\}$/', $line)) {
            $skip_section = false;
            continue;
        }

        if ($skip_section) {
            continue;
        }

        if (1 !== preg_match(
            "/^[\s]*([^\s#;=]+)[\s=]+([\"']?)(.*?)(?<!\\\)([\"']?)[\s;]*$/Su",
            $line,
            $matches
        )) {
            $res = false;
            break;
        }

        $start_quote = & $matches[2];
        $end_quote = & $matches[4];
        if ($start_quote !== $end_quote) {
            $res = false;
            break;
        }

        $param = & $matches[1];
        $value = stripslashes($matches[3]);

        if (is_array($params)) {
            if (false === array_search($param, $params)) {
                continue;
            }
        }

        // нашли параметр
        // error_log(sprintf('file %s:%d : %s => %s (%s)<br>', $path, $linenr, $param, $value, gettype(@$_conf[$param])));
        if (0 === strcasecmp($param, 'include')) {
            // вложенный файл
            $res = confparse($_conf, $sections, $value);
            if (!$res) {
                error_log("ERROR INCLUDE FILE \"$value\" from $path:$linenr");
                $res = false;
                break;
            } else {
                $ret_array = array_merge($ret_array, $res);
            }
        } else {
            /* обычное параметр = значение, не вложенный файл */
            /* проверяем парамет - а мож это массив */
            if (false !== strpos($param, '[')) {
                $tmp = $param;
                $ref = false;
                $r = true;
                $ref_conf = false;

                for ($ar_level = 0; ($ar_level < MAX_ARRAY_PARAM_DEPTH) && $r; $ar_level += 1) {
                    $r = preg_match("/^([^\[]+)\[([\"']?)([^\]]*?)([\"']?)\](.*)/Su", $tmp, $match2);
                    if ($r) {
                        $arr_name = &$match2[1];
                        $key_name = &$match2[3];
                        $remain   = &$match2[5];

                        if ($ar_level === 0) {
                            if (array_key_exists($arr_name, $_conf) && is_array($_conf[$arr_name])) {
                                $ref_conf = &$_conf[$arr_name];
                            } else {
                                // игнорируем необрабатываемый параметр, его нет в site-defaults.php
                                break;
                            }

                            if (!array_key_exists($arr_name, $ret_array)) {
                                $ret_array[$arr_name] = array();
                            }
                            $ref = &$ret_array[$arr_name];
                        }

                        $ref = &$ref[$key_name];

                        if (isset($ref_conf) && is_array($ref_conf)) {
                            if (array_key_exists($key_name, $ref_conf)) {
                                $ref_conf = &$ref_conf[$key_name];
                            } else {
                                unset($ref_conf);
                            }
                        }

                        if (empty($remain)) {
                            if (isset($ref_conf)) {
                                $_v = __settype($ref_conf, $value);
                                if ($_v === null) {
                                    $res = false;
                                    break;
                                }
                                $ref = $_v;
                            } else {
                                $ref = $value;
                            }
                            unset($ref, $ref_conf);
                            break;
                        } else if (is_null($ref)) {
                            $ref = array();
                        }
                        $tmp = 'fakeKey' . $remain;
                    }
                }
            } else {
                /* простое параметр, не массив */
                if (!array_key_exists($param, $_conf)) {
                    // игнорируем необрабатываемый параметр, его нет в site-defaults.php
                    continue;
                }
                /* пробуем установить тип значения с учётом дефолтного $conf[param] */
                $_v = __settype($_conf[$param], $value);
                if ($_v === null) {
                    $res = false;
                    break;
                }
                $ret_array[$param] = $_v;
            }
        }
    } // while eof

    fclose($confile);

    if (!$res) {
        throw new InvalidArgumentException("$path:$linenr => [ $line ]");
    }

    return $ret_array;
} /* confparse() */
/* vim: set expandtab smartindent tabstop=4 shiftwidth=4: */
