<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */
/**
 *
 * @file lib/dl.php
 * @brief Загрузка файла с архива для браузеров не поддерживающих аттрибут download
 *
 * @TODO управление кешированием
 */
require_once './config.inc.php';
require_once './get-cam-urls.php';
require_once './die.inc.php';

setlocale(LC_ALL, 'C');
// hide notices
@ini_set('error_reporting', E_ALL & ~ E_NOTICE);

//- turn off compression on the server
if ($SERVER_SOFT['name_lo'] === 'apache') {
    @apache_setenv('no-gzip', 1);
}
@ini_set('zlib.output_compression', 'Off');

ob_start();

DENY($arch_status);

if (!isset($_REQUEST['cam']) || empty($_REQUEST['cam'])) {
    die412("parameter \"cam\" empty");
}
if (!isset($_REQUEST['f']) || empty($_REQUEST['f'])) {
    die412("parameter \"f\" empty");
}
$camNbr = settype($_REQUEST['cam'], 'integer');
if (!$camNbr) {
    die412("unexpected \"cam\" parameter");
}
$rel_path = trim($_REQUEST['f'], "/ \t\n\r\0\x0B");
$abs_path = getArchFileUrl($camNbr, $rel_path, array('file' => true, 'encode' => false));
$dl_file_name = strtr($rel_path, '/', '--');
$chunk_sz = 1024 * 64;

if (!is_file($abs_path)) {
    die404("file \"$rel_path\" not found");
}

if (!is_readable($abs_path)) {
    die403("file \"$rel_path\" is not readabled");
}

$fsize = filesize($abs_path);
if (!$fsize) {
    die500("file \"$rel_path\" not found");
}

$accel_redirect_header = false;
if ($SERVER_SOFT['name_lo'] === 'apache') {
    if (false !== apache_get_version()) {
        $ap_modules = apache_get_modules();
        if (in_array('mod_xsendfile', $ap_modules)) {
            $accel_redirect_header = 'X-Sendfile: ' . $abs_path;
        }
    }
} else if ($SERVER_SOFT['name_lo'] === 'nginx') {
    $accel_redirect_header = 'X-Accel-Redirect: ' . getArchFileUrl($camNbr, $rel_path, array('encode' => false));
} else if ($SERVER_SOFT['name_lo'] === 'lighttpd') {
    $accel_redirect_header = 'X-Sendfile2: ' . $abs_path; // FIXME "allow-x-send-file" => "enable"
}

while (@ob_end_flush()) {
};

header('Content-Type: application/octet-stream');
header('Content-Disposition: attachment; filename="' . $dl_file_name . '"');
header('Content-Length: ' . $fsize);
header("Pragma: public");
header("Cache-Control: must-revalidate, post-check=0, pre-check=0");

if ($accel_redirect_header) {
    header($accel_redirect_header);
    header('X-Use-Sendfile: yes');
    while (@ob_end_flush()) {
    };
} else {
    header('X-Use-Sendfile: No');
    while (@ob_end_flush()) {
    };

    $fd = @fopen($abs_path, 'rb');
    while (!@feof($fd)) {
        $chunk = @fread($fd, $chunk_sz);
        if ($chunk === false) {
            @fclose($fd);
            exit();
        }
        ob_flush();
        flush();
        if (connection_status() != 0) {
            @fclose($fd);
            exit();
        }
    }
    @fclose($fd);
}
