<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */
/**
 *
 * @file lib/dlpl.php
 * @brief Загрузка файлОВ с архива в архиве ZIP c добавлением playlist.m3u
 *
 * @TODO управление кешированием
 * @TODO ZIP64
 */

namespace Avreg\Playlist;

require_once(__DIR__ . '/config.inc.php');
require_once(__DIR__ . '/die.inc.php');
require_once(__DIR__ . '/playlists/playlist.inc.php');
require_once(__DIR__ . '/get_cams_params.inc.php');
require_once(__DIR__ . '/get-cam-urls.php');

setlocale(LC_ALL, 'C');
// hide notices
// @ini_set('error_reporting', E_ALL & ~ E_NOTICE);

//- turn off compression on the server
if ($SERVER_SOFT['name_lo'] === 'apache') {
    @apache_setenv('no-gzip', 1);
}
@ini_set('zlib.output_compression', 'Off');

ob_start();

DENY($arch_status);

if (!isset($_REQUEST['files']) || empty($_REQUEST['files'])) {
    die412("parameter \"files\" empty");
}

$input_files = json_decode($_REQUEST['files']);
if (null === $input_files ||
    JSON_ERROR_NONE !== json_last_error() ||
    !is_array($input_files) || empty($input_files)) {
    die412("invalid JSON input param");
}

function getCamNr($i)
{
    return (int)($i[2]);
}
$camlist = array_map('\Avreg\Playlist\getCamNr', $input_files);
sort($camlist, SORT_NUMERIC);
$camlist = array_unique($camlist, SORT_NUMERIC);

$cams_params = get_cams_params('text_left', $camlist);

$plgen = getPlaylistGenerator(M3U);
$camNamesCache = array();

$zipFileMap = array();

foreach ($input_files as $tr_info) {
    $udt1 = (int)$tr_info[0];
    $udt2 = (int)$tr_info[1];
    $camNbr = (int)$tr_info[2];
    $sizeKb = (int)$tr_info[3];
    $relPath = $tr_info[4];
    if (!isset($camNamesCache[$camNbr])) {
        $camNamesCache[$camNbr] = shortCamName($camNbr, $cams_params[$camNbr]['text_left']['v']);
    }
    $title = &$camNamesCache[$camNbr];

    $mediaAlias = getCamAvregConf($camNbr, 'media-alias');
    $absPath = getCamAvregConf($camNbr, 'storage-dir') . '/' . $relPath;

    if ($mediaAlias !== '/avreg') {
        $byAvregRelPath = $mediaAlias . '/' . $relPath;
    } else {
        $byAvregRelPath = $relPath;
    }

    // FIXME win path (1), s%/%_% (2)
    $plgen->addTrack([
        'title' => $title,
        'location' => substr($byAvregRelPath, 1),
        'date' => $udt2,
        'duration' => $udt1 - $udt2,
        'sizeKb' => $sizeKb
    ]);

    $zipFilesMap[] = [
        'src' => $absPath,
        'target' => 'avreg' . $byAvregRelPath
    ];
}

$plData = $plgen->gen();
$rootRawMap = array(
    array(
        'target' => 'avreg/playlist.m3u',
        'blob' => array(
            'data' => $plData,
            'length' => strlen($plData)
        )
    )
);

try {
    $zipSink = new Zip\ZipSink($zipFilesMap, $rootRawMap);
} catch (\Exception $e) {
    die500($e->getMessage());
}

$nowts = time();
$now = gmdate('D, d M Y H:i:s', $nowts) . ' GMT';
$filename = 'avreg-playlist-' . date('ymdHis', $nowts) . '.zip';

header('Connection: close');
header('Content-Type: application/zip');
header('Content-Disposition: attachment; filename="' . $filename . '"');
$conlen = $zipSink->getLength();
if ($conlen) {
    header('Content-Length: ' . $conlen);
}
header('Expires: ' . $now);
header('Last-Modified: ' . $now);
header('Cache-Control: private, no-store, no-cache, must-revalidate, pre-check=0, post-check=0, max-age=0'); // HTTP/1.1
header('Pragma: no-cache'); // HTTP/1.0

while (@ob_end_flush()) {
};

try {
    $zipSink->out();
} catch (\Exception $e) {
    die500($e->getMessage());
}
