<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */

/**
 *
 * @file lib/get-cam-urls.php
 * @brief Формирование ссылок на видео (реального времени и архива) с камеры
 *
 * $__JS_VAR__ - если true, то вставляется <script></script>
 */

$cams_subconf = load_profiles_cams_confs(['avregd', 'avreg-site']);

if (!empty($__JS_VAR__)) {
    echo "<script>" . PHP_EOL;
    echo "var CAMS_PROFILES_CONF = " . json_encode($cams_subconf) . ';' . PHP_EOL;
    echo "var MediaAliasDef = '" . $conf['media-alias'] . "';" . PHP_EOL;
    echo "</script>" . PHP_EOL;
    echo "<script src=\"" . $conf['prefix'] . "/lib/js/get-cam-urls.js\"></script>" . PHP_EOL;
}

$__tmp = & $conf['avregd-httpd'];
eval("\$_single_local_avregd_httpd_ = \"$__tmp\";");
unset($__tmp);

function getCamAvregConf($camNbr, $parName)
{
    global $conf, $cams_subconf;

    settype($camNbr, 'integer');

    if ($cams_subconf && @$cams_subconf[$camNbr]) {
        $var = isset($cams_subconf[$camNbr][$parName]) ? $cams_subconf[$camNbr][$parName] : $conf[$parName];
    } else {
        $var = $conf[$parName];
    }

    return $var;
}

/**
 * Возвращает ссылку на файл из архива,
 * с учётом того что файлы в архиве могут быть расположены на неск. блочных устройствах
 */
function getArchFileUrl(
    $camNbr,
    $fileRelPath,
    $options = array(
        'file' => false,
        'encode' => true
    )
)
{
    if ($options['file']) {
        $url = getCamAvregConf($camNbr, 'storage-dir') . '/' . $fileRelPath;
    } else {
        $url = getCamAvregConf($camNbr, 'prefix') . getCamAvregConf($camNbr, 'media-alias') . '/' . $fileRelPath;
    }

    if ($options['encode']) {
        return str_replace(
            ['%3A', '%2F', '%3F', '%26', '%23', '%7E'],
            [':', '/', '?', '&', '#', '~'],
            urlencode($url)
        );
    } else {
        return $url;
    }
} // getArchFileUrl()


/**
 * Возвращает $url если переданный url верный и url avred в случае, если url некорректный
 * @return null|string
 */
function checkUrlParam($url = null)
{
    if (isset($url) && filter_var($url, FILTER_VALIDATE_URL)) {
        return $url;
    }

    return null;
}

/**
 * Функция, которая возвращает AVReg-овую http ссылку на просмотр видео с камеры (AVReg на другом сервере)
 * @param array $conf масив настроек
 * @param int $cam_nr номер камеры
 * @param array $camParams - массив с параметрами камеры
 * @param string $codecOrTech тип медиа: jpeg, mjpeg, hls; default 'mjpeg'
 * @param bool $append_abenc аутентификация пользователя
 * @return string адрес видео с камеры
 */
function get_foreign_avregd_http_url($conf, $cam_nr, $camParams, $codecOrTech = 'mjpeg', $append_abenc = false, $query_string = '')
{
    if (empty($camParams['InetCam_IP'])) {
        return null;
    }

    $url = 'http://' . getCamParamValue($camParams, 'InetCam_IP', 'no_host_in_InetCam_IP') . ':';
    $url .= getCamParamValue($camParams, 'InetCam_http_port', '874');

    $login = getCamParamValue($camParams, 'InetCam_USER', null);
    $password = getCamParamValue($camParams, 'InetCam_PASSWD', null);
    if ($append_abenc && !$login) {
        $append_abenc = false;
    }

    if (!isset($conf['avregd-httpd-paths'])) {
        $conf['avregd-httpd-paths'] = array(
            'mjpeg' => '/avreg-cgi/mjpg/video.cgi?camera=%d',
            'mjpeg-control' => '/avreg-cgi/mjpg/control.cgi?camera=%d&connection=close',
            'jpeg' => '/avreg-cgi/jpg/image.cgi?camera=%d&connection=close',
            'hls' => '/hls/%d/master.m3u8',
            'rec' => '/avreg-cgi/operator/rec.cgi?camera=%d&connection=close'
        );
    }

    if (@!empty($conf['avregd-httpd-paths'][$codecOrTech])) {
        $url .= sprintf($conf['avregd-httpd-paths'][$codecOrTech], $cam_nr);
    }

    if (!empty($query_string) || $append_abenc) {
        if (false !== strrpos($url, '?')) {
            $url .= '&';
        } else {
            $url .= '?';
        }
    }

    if (!empty($query_string)) {
        $url .= $query_string;
        if ($append_abenc) {
            $url .= '&';
        }
    }

    if ($append_abenc) {
        $url .= 'ab=' . urlencode(base64_encode($login . ':' . $password));
    }
    return $url;
}

/**
 * Функция, которая возвращает AVReg-овую rtp ссылку на просмотр видео с камеры (AVReg на этом сервере)
 * @param array $conf масив настроек
 * @param int $cam_nr номер камеры
 * @param string $query_string доп. параметры
 * @return string адрес видео с камеры
 */
function get_this_avreg_rtsp_url($conf, $cam_nr, $query_string = '')
{
    $uri = '';

    $cams_subconf = & $GLOBALS['cams_subconf'];

    if ($cams_subconf && isset($cams_subconf[$cam_nr])
        && !empty($cams_subconf[$cam_nr]['rtsp-port'])) {
        $rtsp_port = intval($cams_subconf[$cam_nr]['rtsp-port']);
    } else if (isset($conf['rtsp-port'])) {
        $rtsp_port = intval($conf['rtsp-port']);
    } else {
        $rtsp_port = 8554;
    }

    $uri = sprintf(
        'rtsp://%s:%s@%s:%u/avreg-media/%u/media.amp',
        $_SERVER['REMOTE_USER'],
        $_SERVER['PHP_AUTH_PW'],
        $_SERVER['SERVER_NAME'],
        $rtsp_port,
        $cam_nr
    );

    if (!empty($query_string)) {
        $uri .= $query_string;
    }

    return $uri;
}

function is_same_origin($url1, $url2)
{
    $port1 = false;
    $port2 = true;

    $u1 = parse_url ($url1);
    $u2 = parse_url ($url2);

    // error_log(print_r($u1, true));
    // error_log(print_r($u2, true));

    if (!$u1 || !@$u1['scheme'] || !@$u1['host'] ||
        !$u2 || !@$u2['scheme'] || !@$u2['host']) {
        throw new TypeError ("Unexpected url1 \"$url1\" and/or url2 \"$url2\"");
    }

    if ($u1['scheme'] !== $u2['scheme']) {
        return false;
    }

    $host1 = gethostbyname($u1['host']);
    $host2 = gethostbyname($u1['host']);
    if ($host1 !== $host2) {
        return false;
    }

    if (is_integer(@$u1['port'])) {
        $port1 = $u1['port'];
    } else if ($u1['scheme'] === 'https') {
        $port1 = 443;
    } else if ($u1['scheme'] === 'http') {
        $port1 = 80;
    }

    if (is_integer(@$u2['port'])) {
        $port2 = $u2['port'];
    } else if ($u2['scheme'] === 'https') {
        $port2 = 443;
    } else if ($u2['scheme'] === 'http') {
        $port2 = 80;
    }

    return ($port2 === $port1);
}

/**
 * Функция, которая возвращает AVReg-овую http ссылку на просмотр видео с камеры (AVReg на этом сервере)
 * @param array $conf масив настроек
 * @param int $cam_nr номер камеры
 * @param string $codecOrTech тип медиа: jpeg, mjpeg, hls; default 'mjpeg'
 * @param bool|string $append_abenc аутентификация пользователя, or "auto" to check same origin
 * @param string $query_string доп. параметры
 * @return string адрес видео с камеры
 */
function get_this_avregd_http_url($conf, $cam_nr, $codecOrTech = 'mjpeg', $append_abenc = 'auto', $query_string = '')
{
    $cams_subconf = & $GLOBALS['cams_subconf'];

    if ($cams_subconf && isset($cams_subconf[$cam_nr])
        && !empty($cams_subconf[$cam_nr]['avregd-httpd'])) {
        $_a = & $cams_subconf[$cam_nr]['avregd-httpd'];
        eval("\$url = \"$_a\";");
    } else {
        $url = $GLOBALS['_single_local_avregd_httpd_'];
    }

    if ($append_abenc === 'auto') {
        if ($GLOBALS['GuestAuth']) {
            $append_abenc = true;
        } else {
            $tmp = ($GLOBALS['SSL_ACCESS_USED'] ? 'https://' : 'http://') . $_SERVER['HTTP_HOST'];
            $append_abenc = !is_same_origin($url, $tmp);
        }
    }
    if ($append_abenc) {
        $authUser = (!empty($GLOBALS['user_info']['USER'])) ? $GLOBALS['user_info']['USER'] : null;
        if ($append_abenc && !$authUser) {
            $append_abenc = false;
        }
    }

    if (!isset($conf['avregd-httpd-paths'])) {
        $conf['avregd-httpd-paths'] = array(
            'mjpeg' => '/avreg-cgi/mjpg/video.cgi?camera=%d',
            'mjpeg-control' => '/avreg-cgi/mjpg/control.cgi?camera=%d&connection=close',
            'jpeg' => '/avreg-cgi/jpg/image.cgi?camera=%d&connection=close',
            'hls' => '/hls/%d/master.m3u8',
            'rec' => '/avreg-cgi/operator/rec.cgi?camera=%d&connection=close'
        );
    }

    if (@!empty($conf['avregd-httpd-paths'][$codecOrTech])) {
        $url .= sprintf($conf['avregd-httpd-paths'][$codecOrTech], $cam_nr);
    }

    if (!empty($query_string) || $append_abenc) {
        if (false !== strrpos($url, '?')) {
            $url .= '&';
        } else {
            $url .= '?';
        }
    }

    if (!empty($query_string)) {
        $url .= $query_string;
        if ($append_abenc) {
            $url .= '&';
        }
    }

    if ($append_abenc) {
        $url .= 'ab=' . urlencode(base64_encode($authUser . ':' . $_SERVER['PHP_AUTH_PW']));
    }
    return $url;
}

/***
 * Возвращает url медиа-потоков камеры из настроек аврега.
 *
 * @param array $cam_params      масив настроек камеры cams_params[$cam_nr] get_cams_params(),
 *                               необходимый для постройки URL
 * @param string $pref_proto_csv CSV-список протоколов в порядке предпочтения
 * @param string $query          query: param1=val1&param2=val2&...
 * @return null|string           адрес видео с камеры
 */
function build_cam_url($cam_params, $pref_proto_csv = null, $query = null)
{
    if (@empty($cam_params['video_src']['v'])) {
        return null;
    }

    if (is_empty_var($pref_proto_csv)) {
        $proto_a = array($cam_params['video_src']['v']);
    } else {
        $proto_a = array_map('trim', explode(',', $pref_proto_csv));
        $proto_a = array_map('strtolower', $proto_a);
    }

    foreach ($proto_a as &$proto_pref) {
        $req = '/';
        $port = '';
        $auth = '';
        $__query = '';
        if ($proto_pref === 'rtsp') {
            if (!@empty($cam_params['InetCam_rtsp_port']['v']) && (int)$cam_params['InetCam_rtsp_port']['v'] !== 554) {
                $port = $cam_params['InetCam_rtsp_port']['v'];
            }
            if (!@empty($cam_params['rtsp_play']['v'])) {
                $req =& $cam_params['rtsp_play']['v'];
            }
        } elseif ($proto_pref === 'http') {
            if (!@empty($cam_params['InetCam_http_port']['v']) && (int)$cam_params['InetCam_http_port']['v'] !== 80) {
                $port = $cam_params['InetCam_http_port']['v'];
            }
            if (!@empty($cam_params['V.http_get']['v'])) {
                $req =& $cam_params['V.http_get']['v'];
            }
        } else {
            return null;
        }
        if (@empty($cam_params['InetCam_IP']['v'])) {
            return null;
        }

        if (!@empty($cam_params['InetCam_USER']['v'])) {
            $pwd = '';
            if (!@empty($cam_params['InetCam_USER']['v'])) {
                $pwd =& $cam_params['InetCam_PASSWD']['v'];
            }
            $auth = $cam_params['InetCam_USER']['v'] . ':' . $pwd . '@';
        }

        if (!@empty($query)) {
            if (false !== strrpos($req, '?')) {
                $__query = '&' . $query;
            } else {
                $__query = '?' . $query;
            }
        }

        $url = $proto_pref . '://' . $auth . $cam_params['InetCam_IP']['v'];
        if (!empty($port))
            $url .= ':' . $port;
        $url .=  $req . $__query;
        if (filter_var($url, FILTER_VALIDATE_URL)) {
            return $url;
        }
    }
    return null;
} /* build_cam_url() */

/***
 * Строит альтернативный URL $src_template = ...
 *   - {cam_nr}[:avregd[:http[:{req_params}]]
 *   - {cam_nr}:camera[:(rtsp|http)[:{req_params}]]
 *   - (http|rtsp)://[login:password@]host:port/path?query
 */
function get_cam_url($conf, $cam_nr, $cams_params, $src_template = null)
{
    $target_cam_nr = false;
    $remote_avregd = false;
    $query_str = null;
    $ret = [
        'is_avregd_cam' => false,
        'proto' => null,
        'codecOrTech' => null,
        'hasAudio' => null,
        'urls' => [
            'stream' => null,
            'streaming_ctrl' => null,
            'snapshot' => null
        ]
    ];
    $avregCodecOrTechs = array(
        'mjpeg' => 'http',
        'hls' => 'http'
    );

    $protos = array('http', 'rtsp');

    $url = null;
    if (is_empty_var($src_template)) {
        $src_template = (string)$cam_nr;
    }

    $a = explode(':', $src_template);
    if (is_array($a) && count($a) >= 1) {
        switch (strtolower($a[0])) {
            case 'http':
            case 'rtsp':
                $ret['proto'] = strtolower($a[0]);
                switch($ret['proto']) {
                    case 'http':
                        if (false !== strpos($src_template, '.m3u8')) {
                            $ret['codecOrTech'] = 'hls';
                        } else {
                            $ret['codecOrTech'] = 'mjpeg';
                        }
                        break;
                    case 'rtsp':
                        $ret['codecOrTech'] = 'rtsp';
                        break;
                }
                $url = checkUrlParam($src_template);
                break;
            default:
                if ($a[0] === 'self' || $a[0] === 'me' || $a[0] === 'own') {
                    $a[0] = $cam_nr;
                } else if ($a[0] === 'prev') {
                    $a[0] = $cam_nr - 1;
                } else if ($a[0] === 'prev2') {
                    $a[0] = $cam_nr - 2;
                } else if ($a[0] === 'next') {
                    $a[0] = $cam_nr + 1;
                } else if ($a[0] === 'next2') {
                    $a[0] = $cam_nr + 2;
                }

                /* expect cam number */
                if (settype($a[0], 'integer')) {
                    $target_cam_nr = $a[0];
                    array_shift($a);

                    if ($target_cam_nr < 0) {
                        $remote_avregd = true;
                        $target_cam_nr = -$target_cam_nr;
                    }

                    if (!@empty($a[0])) {
                        $tmp = $a[0];
                        if (0 === strpos($tmp, 'avreg')) {
                            $ret['is_avregd_cam'] = $target_cam_nr;
                            array_shift($a);
                        } else if (0 === strpos($tmp, 'cam')) {
                            array_shift($a);
                        }
                    } else {
                        $ret['is_avregd_cam'] = $target_cam_nr;
                        $ret['codecOrTech'] = 'mjpeg';
                        $ret['proto'] = $avregCodecOrTechs['mjpeg'];
                    }

                    if (!@empty($a[0])) {
                        $tmp = $a[0];
                        $tmp_lowcase = strtolower($tmp);
                        if (in_array($tmp_lowcase, $protos)) {
                            $ret['proto'] = $tmp_lowcase;
                            array_shift($a);
                        }
                    }

                    if (!@empty($a[0]) && false === @strpos($a[0], '=')) {
                        $tmp = $a[0];
                        if (isset($avregCodecOrTechs[$tmp])) {
                            $ret['is_avregd_cam'] = $target_cam_nr;
                            $ret['codecOrTech'] = $tmp;
                            $ret['proto'] = $avregCodecOrTechs[$tmp];
                            array_shift($a);
                        }
                    } else if ($ret['is_avregd_cam'] && $ret['proto'] !== 'rtsp') {
                        // $ret['is_avregd_cam'] = $target_cam_nr;
                        $ret['codecOrTech'] = 'mjpeg';
                        $ret['proto'] = $avregCodecOrTechs['mjpeg'];
                    }

                    if (!@empty($a[0])) {
                        $tmp = $a[0];
                        if (false !== @strpos($tmp, '=')) {
                            $query_str = $tmp;
                        }
                    }

                    if ($remote_avregd) {
                        // -cam_nr:avreg:...
                        $camParams = &$cams_params[$cam_nr];
                        if ($camParams['work']['v'] && $camParams['allow_networks']['v']
                            && !empty($camParams['video_src']['v'])) {
                            if ($ret['is_avregd_cam']) {
                                if ($ret['codecOrTech'] === 'hls') {
                                    if (!empty($camParams['audio_src']['v']) &&
                                        (empty($query_str) || false === strpos($query_str, 'audio=0'))) {
                                        $ret['hasAudio'] = true;
                                    }
                                }
                                $url = get_foreign_avregd_http_url(
                                    $conf, $target_cam_nr, $camParams, $ret['codecOrTech'], true, $query_str
                                );
                                $ret['urls']['snapshot'] = get_foreign_avregd_http_url(
                                    $conf, $target_cam_nr, $camParams, 'jpeg', true
                                );
                                if ($ret['codecOrTech'] === 'mjpeg') {
                                    $ret['urls']['streaming_ctrl'] = get_foreign_avregd_http_url(
                                        $conf, $target_cam_nr, $camParams, 'mjpeg-control', true
                                    );
                                }
                            } else {
                                // only avreg http url allow, not a cameras
                                return false;
                            }
                        }
                    } else {
                        // this server or camera at this server
                        if (array_key_exists($target_cam_nr, $cams_params)) {
                            $camParams = &$cams_params[$target_cam_nr];
                            if ($camParams['work']['v'] && $camParams['allow_networks']['v']
                                && !empty($camParams['video_src']['v'])) {
                                if ($ret['is_avregd_cam']) {
                                    if ($ret['codecOrTech'] === 'hls') {
                                        if (!empty($camParams['audio_src']['v']) &&
                                            (empty($query_str) || false === strpos($query_str, 'audio=0'))) {
                                            $ret['hasAudio'] = true;
                                        }
                                    }
                                    if ($ret['proto'] === 'rtsp') {
                                        $url = get_this_avreg_rtsp_url($conf, $target_cam_nr, $query_str);
                                    } else {
                                        $url = get_this_avregd_http_url($conf, $target_cam_nr, $ret['codecOrTech'], 'auto', $query_str);
                                    }
                                } else {
                                    // $url_src = 'camera';
                                    if (!$ret['proto']) {
                                        $ret['proto'] = $cams_params[$target_cam_nr]['video_src']['v'];
                                    }
                                    if ($ret['proto'] === 'http') {
                                        $ret['codecOrTech'] = 'mjpeg';
                                    } else if ($ret['proto'] === 'rtsp') {
                                        $ret['codecOrTech'] = 'rtsp';
                                    }
                                    $url = build_cam_url($cams_params[$target_cam_nr], $ret['proto'], $query_str);
                                }
                            }
                        }
                    }
                }
                break;
        }
    }

    if (!empty($url)) {
        if ($ret['is_avregd_cam']) {
            if (!$ret['urls']['snapshot']) {
                $ret['urls']['snapshot'] = get_this_avregd_http_url($conf, $target_cam_nr, 'jpeg');
            }

            if ($ret['codecOrTech'] === 'mjpeg' && !$ret['urls']['streaming_ctrl']) {
                $ret['urls']['streaming_ctrl'] = get_this_avregd_http_url($conf, $target_cam_nr, 'mjpeg-control');
            }
        }
        $ret['urls']['stream'] = $url;
        return $ret;
    } else {
        return false;
    }
} /* get_cam_url() */
