<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */
/**
 * @file lib/get-events.php
 * @brief get EVENTS table
 *
 * @in cams[]     array of int or string (list) or does not set
 * @in date-range "YYYY-MM-DD - YYYY-MM-DD" or "YYYY-MM-DD..YYYY-MM-DD" or date-from + date-to
 * @in date-from  "YYYY-MM-DD"
 * @in date-to    "YYYY-MM-DD"
 * @in time-from  "(H)H:MM"
 * @in time-to    "(H)H:MM"
 * @in weekdays[] Mon(0)..Sun(6)
 * @in events[]
 * @in time-slices-mode
 * @in limit
 * @in offset
 *
 * @return JSON { status => '', description => '' }
 */

require('../head-xhr.inc.php');
DENY($arch_status); // FIXME TODO возвратит html вместо json, м.б. неудобно

$errors = array();

/*** check input params ***/
$cams = false;
if (isset($_REQUEST['cams'])) {
    $cams = request_int_array('cams', $ALLOW_CAMS);
} else {
    require 'get_cams_params.inc.php';
    $rec_cams = get_recordable_cameras();
    if (count($rec_cams) > 0) {
        $cams = array_keys($rec_cams);
    }
}
if (!is_array($cams) || count($cams) === 0) {
    $errors[] = 'cams[]: empty or unexpected value or do not have recordable cams';
}

$time_slices_mode = false;
$weekdays = false;
if (isset($_REQUEST['time-slices-mode']) &&
    filter_var($_REQUEST['time-slices-mode'], FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE)) {

    $weekdays = request_int_array('weekdays', range(0,6));
    if (!is_array($weekdays) || count($weekdays) === 0) {
        $errors[] = 'weekdays[]: empty or unexpected value';
    } else {
        $time_slices_mode = true;
    }
}

$events = request_int_array('events', range(1,32));
if (!is_array($events) || count($events) === 0) {
    $errors[] = 'events[]: empty or unexpected value';
}
if ((in_array(23, $events) || in_array(32, $events)) && !in_array(12, $events)) {
    $events[] = 12;
}

$year_from = $month_from = $day_from = $year_to = $month_to = $day_to = false;
if (isset($_REQUEST['date-range']) && is_string($_REQUEST['date-range'])) {
    if (preg_match("/(\d{4})-(\d{2})-(\d{2})[\s\.-]*(\d{4})-(\d{2})-(\d{2})/", $_REQUEST['date-range'], $matches)) {
        $year_from = (int)$matches[1];
        $month_from = (int)$matches[2];
        $day_from = (int)$matches[3];
        $year_to = (int)$matches[4];
        $month_to = (int)$matches[5];
        $day_to = (int)$matches[6];
    } else {
        $errors[] = 'date-range: invalid value ' . $_REQUEST['date-range'];
    }
} else {
    if (isset($_REQUEST['date-from'])) {
        if (preg_match("/(\d{4})-(\d{2})-(\d{2})/", $_REQUEST['date-from'], $matches)) {
            $year_from = (int)$matches[1];
            $month_from = (int)$matches[2];
            $day_from = (int)$matches[3];
        } else {
            $errors[] = 'date-from: invalid value ' . $_REQUEST['date-from'];
        }
    }
    if (isset($_REQUEST['date-to'])) {
        if (preg_match("/(\d{4})-(\d{2})-(\d{2})/", $_REQUEST['date-to'], $matches)) {
            $year_to = (int)$matches[1];
            $month_to = (int)$matches[2];
            $day_to = (int)$matches[3];
        } else {
            $errors[] = 'date-to: invalid value ' . $_REQUEST['date-to'];
        }
    }
}

$hour_from = $minute_from = $hour_to = $minute_to = false;
if (isset($_REQUEST['time-from'])) {
    if (preg_match("/(\d{1,2}):(\d{2})/", $_REQUEST['time-from'], $matches)) {
        $hour_from = (int)$matches[1];
        $minute_from = (int)$matches[2];
    } else {
        $errors[] = 'time-from: invalid value ' . $_REQUEST['time-from'];
    }
}
if (isset($_REQUEST['time-to'])) {
    if (preg_match("/(\d{1,2}):(\d{2})/", $_REQUEST['time-to'], $matches)) {
        $hour_to = (int)$matches[1];
        $minute_to = (int)$matches[2];
    } else {
        $errors[] = 'time-to: invalid value ' . $_REQUEST['time-to'];
    }
}

$dateTimeRange = array(
    'from' => array($year_from, $month_from, $day_from, $hour_from, $minute_from),
    'to' => array($year_to, $month_to, $day_to, $hour_to, $minute_to),
);

if (settype($limit, 'integer') && $limit > 0 && settype($offset, 'integer')) {
    // error_log("limit=" . print_r($limit, true) . '  offset=' . print_r($offset, true));
    $reduceRows = array(
        'limit' => (int)$limit,
        'offset' => @empty($offset) ? 0 : (int)$offset
    );
} else {
    $reduceRows = null;
}

if (count($errors) > 0) {
    $ret = array(
        'status' => 'error',
        'messages' => $errors,
    );
    echo json_encode($ret);
    exit();
}

/*** perform sql query ***/
$result = $adb->eventsSelect(
    $cams,
    $dateTimeRange,
    $events,
    isset($weekdays) ? $weekdays : array(),
    $time_slices_mode,
    $reduceRows
);

// file_put_contents('/tmp/request.txt', print_r($_REQUEST, true));
// file_put_contents('/tmp/query.txt', print_r($__EVENTS_QUERY_INFO, true));

/*** prepare result ***/
echo json_encode($result);
