<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */
/**
 * @file /lib/_playlist.php
 * @brief Страница формирования плейлиста
 *
 * POST параметры.
 *  cams[] - массив номеров камер
 *  ftypes[] - массив типов медиа (12 - a+v, 23 -video, 32 - аudio)
 *  time-slices-mode - true|on|yes|1 если интервалы
 *  year{1,2},month{1,2},day{1,2},hour{1,2},minute{1,2} - начало(1) и окончание(2)
 *  pl_fmt - формат плейлиста: M3U, XSPF, JSPF, TXT
 *  weekdays[]: - массив дней недели, исп. если time-slices-mode is true (интервалы)
 */

ob_start();

$USE_SESSION = true;
require '../lib/config.inc.php';
require '../lib/get-cam-urls.php';

DENY($arch_status);


function input_data_invalid($err_code)
{
    if (@empty($_SERVER['HTTP_X_REQUESTED_WITH'])) {
        // ajax
        header('Location: ' . $GLOBALS['conf']['prefix'] . '/offline/playlist.php');
    }
    $_SESSION['error'] = $err_code;
    exit;
}

function xmlDate($timestamp)
{
    $main_date = date("Y-m-d\TH:i:s", $timestamp);
    $tz = date("O", $timestamp);
    $tz = substr_replace($tz, ':', 3, 0);
    return $main_date . $tz;
}

/* validate input form's data */

if (!is_array($cams) || (count($cams) === 0)) {
    input_data_invalid('cams');
}
foreach ($cams as &$value) {
    if (!settype($value, 'int')) {
        input_data_invalid('cams');
    }
}
if (is_array($ALLOW_CAMS) && count($ALLOW_CAMS) > 0) {
    $a = array_intersect($ALLOW_CAMS, $cams);
    $cams = array_values($a);
    if (count($cams) === 0) {
        input_data_invalid('cams');
    }
}

if (!is_array($ftypes) || (count($ftypes) === 0)) {
    input_data_invalid('ftypes');
}
foreach ($cams as &$value) {
    if (!settype($value, 'int')) {
        input_data_invalid('ftypes');
    }
}

$int_vals = array(
    'year1',
    'month1',
    'day1',
    'hour1',
    'minute1',
    'year2',
    'month2',
    'day2',
    'hour2',
    'minute2'
);
foreach ($int_vals as &$value) {
    if (!isset($GLOBALS[$value])) {
        input_data_invalid($value);
    }
    if (!settype($GLOBALS[$value], 'int')) {
        input_data_invalid($value);
    }
    $_SESSION[$value] = $GLOBALS[$value];
}

$to_int = function ($value) {
    if (!settype($value, 'int')) {
        die('crack or hack?');
    }
    return $value;
};

$time_slices_mode = false;
if (isset($_REQUEST['time-slices-mode']) &&
    filter_var($_REQUEST['time-slices-mode'], FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE)) {
    if (isset($weekdays) && is_array($weekdays) && count($weekdays) > 0) {
        $weekdays = array_map($to_int, $weekdays);
    }
    if (!isset($weekdays) || !is_array($weekdays) || count($weekdays) === 0) {
        input_data_invalid('weekdays[]');
    } else {
        $time_slices_mode = true;
    }
    $_SESSION['weekdays'] = $weekdays;
}

if (empty($pl_fmt)) {
    input_data_invalid('pl_fmt');
}

$pl_fmt = strtoupper($pl_fmt);

if ('XSPF' !== $pl_fmt &&
    'JSPF' !== $pl_fmt &&
    'M3U'  !== $pl_fmt &&
    'TXT'  !== $pl_fmt) {
    input_data_invalid('pl_fmt="' . $pl_fmt . '"');
}

$events = & $ftypes;
$_cams_csv = implode(',', $cams);

$date = array(
    'from' => array($year1, $month1, $day1, $hour1, $minute1),
    'to' => array($year2, $month2, $day2, $hour2, $minute2),
);
// error_log(print_r($date, true));

//Если ищем видео - добавляем в поиск video+audio (EVT_ID==12)
$is_video = false;
foreach ($events as $val) {
    if ($val == 23) {
        $is_video = true;
    }
}
if ($is_video) {
    array_push($events, 12);
}

$res_array = $adb->eventsSelect(
    $cams,
    $date,
    $events,
    isset($weekdays) ? $weekdays : array(),
    $time_slices_mode
);

if ($conf['debug']) {
    $_SESSION['sql'] = $query;
}
$num_rows = @count($res_array);

if (!$num_rows) {
    input_data_invalid('0');
}

/* отправляем http headers */
while (@ob_end_clean()) {
    ;
}

// Don't use cache (required for Opera)
$now_ts = time();
$now = gmdate('D, d M Y H:i:s', $now_ts) . ' GMT';
$now_XML = xmlDate($now_ts);
header('Expires: ' . $now);
header('Last-Modified: ' . $now);
header('Cache-Control: no-store, no-cache, must-revalidate, pre-check=0, post-check=0, max-age=0'); // HTTP/1.1
header('Pragma: no-cache'); // HTTP/1.0

date_default_timezone_set('UTC');

function getDayName($n)
{
    return $GLOBALS['day_of_week'][$n];
}

$ua = @strtolower($_SERVER['HTTP_USER_AGENT']);
$CRLF = "\r\n";
if ($MSIE /* only win */ || $MS_EDGE || false !== strpos($ua, 'windows')) {
    $platform = 'win';
} else {
    if (false !== strpos($ua, 'linux') ||
        false !== strpos($ua, 'solaris') ||
        false !== strpos($ua, 'bsd') ||
        false !== strpos($ua, 'mac os x')
    ) {
        $platform = 'nix';
        if ($lineending == 'AUTO' || $lineending == 'CR') {
            $CRLF = "\n";
        }
    } else {
        $platform = 'other';
    }
}

/*** TODO REMOVE LEGACY
 * вычисляем протокол доступа к медиа-файлам
на основании адреса клиента и правил в конфиге
require_once($wwwdir . 'lib/utils-inet.php');
$remote_ip = ip2long($_SERVER['REMOTE_ADDR']);

$_ipacl_list = array_keys($conf["murl-pre-$platform"]);
foreach ($_ipacl_list as &$_ipacl_str) {
    $_ipacl = avreg_inet_network($_ipacl_str);
    if ($_ipacl === false) {
        // неправильно задан IP ACL
        break;
    }
    if (true === avreg_ipv4_cmp($remote_ip, -1, $_ipacl['addr'], $_ipacl['mask'])) {
        $Media-Url-Pref = & $conf["murl-pre-$platform"][$_ipacl_str];
        break;
    }
}
if (!isset($Media-Url-Pref) ) {
    // не нашли совпадения по ACL
    $Media-Url-Pref = ($GLOBALS['SSL_ACCESS_USED'] ? 'https://' : 'http://') . $_SERVER['SERVER_NAME'];
    $Media-Url-Pref .= $GLOBALS['SERVER_PORT_STR'];
    $Media-Url-Pref .= $conf['prefix'] . $conf['media - alias'];
}
*/

$fname = "avreg_cams-$_cams_csv";
if ($pl_fmt === 'XSPF' || $pl_fmt === 'JSPF') {
    if ($pl_fmt === 'XSPF') {
        header('Content-Type: application/xspf+xml');
        header("Content-Disposition: attachment; filename=\"$fname.xspf\"");
        echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>$CRLF";
        echo "<playlist version=\"0\" xmlns=\"http://xspf.org/ns/0/\">$CRLF";
    } else {
        // JSPF
        header('Content-Type: application/json');
        header("Content-Disposition: attachment; filename=\"$fname.json\"");
    }

    $pjson = array();
    $pjson['playlist'] = array();
    $p = &$pjson['playlist'];
    if ($time_slices_mode) {
        $p['title'] = sprintf(
            '[%s - %s], [%s], [%s-%s]',
            substr($GLOBALS['__EVENTS_QUERY_INFO']['timebegin'], 0, -9),
            substr($GLOBALS['__EVENTS_QUERY_INFO']['timeend'], 0, -9),
            implode(',', array_map("getDayName", $GLOBALS['__EVENTS_QUERY_INFO']['weekdays'])),
            substr($GLOBALS['__EVENTS_QUERY_INFO']['time_in_day_begin'], 0, -3),
            substr($GLOBALS['__EVENTS_QUERY_INFO']['time_in_day_end'], 0, -3)
        );
    } else {
        $p['title'] = sprintf(
            '[%s - %s]',
            substr($GLOBALS['__EVENTS_QUERY_INFO']['timebegin'], 0, -3),
            substr($GLOBALS['__EVENTS_QUERY_INFO']['timeend'], 0, -3)
        );
    }
    $p['creator'] = "cams: [$_cams_csv]";
    $p['annotation'] = $GLOBALS['__EVENTS_QUERY_INFO']['query'];
    $p['info'] = 'http://avreg.net';
    $p['date'] = $now_XML;

    $p['track'] = array();

    if ($pl_fmt === 'XSPF') {
        printf(
            "\t<title>%s</title>$CRLF",
            $p['title']
        );
        echo "\t<creator>" . $p['creator'] . "</creator>$CRLF";
        echo "\t<annotation>" . $p['annotation'] . "</annotation>$CRLF";
        echo "\t<info>" . $p['info'] . "</info>$CRLF";
        echo "\t<date>" . $p['date'] . "</date>$CRLF";
        echo "\t<trackList>$CRLF";
    }
} elseif ($pl_fmt === 'M3U') {
    header('Content-Type: audio/x-mpegurl');
    header("Content-Disposition: attachment; filename=\"$fname.m3u\"");
    echo "#EXTM3U$CRLF";
} else {
    header('Content-Type: text/plain');
    header("Content-Disposition: attachment; filename=\"$fname.txt\"");
}

while (@ob_end_flush()) {
    ;
}

for ($i = 1; $i <= $num_rows; $i++) {
    $row = & $res_array[$i - 1];
    $location = $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['HTTP_HOST'] . getArchFileUrl((int)$row['CAM_NR'], $row['EVT_CONT']);

    if ($row['EVT_ID'] == 23) {
        $_media_str = '(video)';
    } else {
        if ($row['EVT_ID'] == 32) {
            $_media_str = '(audio)';
        } else {
            if ($row['EVT_ID'] == 12) {
                $_media_str = '(video+audio)';
            } else {
                $_media_str = 'unknown';
            }
        }
    }

    $duration = (int)$row['UDT1'] - (int)$row['UDT2'];
    if ($pl_fmt === 'XSPF' || $pl_fmt === 'JSPF') {
        $t['title'] = strftime('%a, %d %b %H:%M:%S', (int)$row['UDT2']);
        $t['duration'] = $duration * 1000;
        $t['location'] = array($location);
        $t['creator'] = sprintf("cam #%02u %s", (int)$row['CAM_NR'], $_media_str);
        $t['annotation'] = filesizeHuman((int)$row['FILESZ_KB']) . ', ' . $row['FRAMES'] . ' frames';

        if ($pl_fmt === 'XSPF') {
            echo "\t\t<track>$CRLF";
            echo "\t\t\t<location>" . $location . "</location>$CRLF";
            echo "\t\t\t<title>" . $t['title'] . "</title>$CRLF";
            echo "\t\t\t<creator>" . $t['creator'] . "</creator>$CRLF";
            echo "\t\t\t<annotation>" . $t['annotation'] . "</annotation>$CRLF";
            echo "\t\t\t<duration>" . $t['duration'] . "</duration>$CRLF";
            // echo "\t\t\t<album>". $row['EVT_ID'] ."</album>$CRLF";
            echo "\t\t\t<trackNum>$i</trackNum>$CRLF";
            echo "\t\t</track>$CRLF";
        } else {
            array_push($p['track'], $t);
        }
    } else {
        if ($pl_fmt === 'M3U') {
            $title = sprintf('cam #%02u %s - ', (int)$row['CAM_NR'], $_media_str) . date(
                'Y-m-d H:i:s',
                (int)$row['UDT2']
            );
            echo "#EXTINF:$duration,$title$CRLF";
        }
        echo "$location$CRLF";
    }
}

if ($pl_fmt === 'XSPF') {
    echo "\t</trackList>$CRLF";
    echo "</playlist>$CRLF";
} elseif ($pl_fmt === 'JSPF') {
    echo json_encode($pjson, JSON_PRETTY_PRINT);
}

if (isset($_SESSION) && isset($_SESSION['error'])) {
    unset($_SESSION['error']);
}
