<?php
/**
 * Copyright (C) 2004-2021 "Setevye informacionnye sistemy", Ltd (Russia)
 *                          en: "Network Information Systems"
 */

namespace Avreg;

class Cache
{
    const DEF_LOCK_EXPIRE = 60; // 1 minute

    private $memcache;
    private $cache_id = '';
    private $locked = ':lock';

    private $pconnect = true;
    private $server = 'localhost';
    private $port = 11211;
    private $debug = false;

    public function __construct($cache_id = '', $debug = false)
    {
        $this->cache_id = $cache_id;
        $this->debug    = $debug;

        $this->memcache = new \Memcached;
        $this->memcache->addServer($this->server, $this->port);

        if ($this->debug) {
            error_log(sprintf('%s(%s, %s)', __METHOD__, $this->cache_id, (string)$debug));
        }
    }

    public function __destruct()
    {
        $this->memcache->quit();
        if ($this->debug) {
            error_log(sprintf('%s()', $this->cache_id, __METHOD__));
        }
    }

    protected function keyName($key)
    {
        if (empty($this->cache_id)) {
            return $key;
        } else {
            return ($this->cache_id . ':' . $key);
        }
    }

    protected function lockName($key)
    {
        $_key = '';
        if (!empty($this->cache_id)) {
            $_key .= $this->cache_id . ':';
        }
        $_key .= $key . ':lock';

        return $_key;
    }

    public function get($key)
    {
        $_key = $this->keyName($key);
        $data = $this->memcache->get($_key);
        if ($this->debug) {
            error_log(sprintf('%s("%s") -> %s%s', __METHOD__, $_key, gettype($data), @empty($data) ? '(empty)' : ''));
        }
        return $data;
    }

    public function lock($key, $time = 10)
    {
        $key_lock = $this->lockName($key);
        $this->memcache->set($key_lock, 1, $time);
    }

    public function set($key, $value, $time = 0)
    {
        $_key = $this->keyName($key);
        $ret_set = $this->memcache->set($_key, $value, $time);
        if ($this->debug) {
            error_log(sprintf('%s("%s, $value, %s, %d") -> %s', __METHOD__, $_key, (string)$compress, $time, $ret_set));
        }
        $key_lock = $this->lockName($key);
        $ret_del_lock = $this->delete($key_lock);
        if ($this->debug) {
            error_log(sprintf('%s() delete("%s") -> %s', __METHOD__, $key_lock, $ret_del_lock));
        }
        return $ret_set;
    }

    public function check($key)
    {
        $key_lock = $this->lockName($key);
        // return true; // temporarily cache disabling FIXME FIXME
        return (bool)$this->memcache->get($key_lock);
    }

    public function delete($key)
    {
        $_key = $this->keyName($key);
        $ret = $this->memcache->delete($_key);
        if ($this->debug) {
            error_log(sprintf('%s("%s") -> %s', __METHOD__, $_key, $ret));
        }
        return $ret;
    }

    public function flush()
    {
        $ret = $this->memcache->flush();
        if ($this->debug) {
            error_log(sprintf('%s("") -> %s', __METHOD__, $ret));
        }
        return $ret;
    }

    public function lockAtomicWait ($key, $wait_timeout = 5)
    {
        $_key = $this->keyName($key);
        while (true) {
            $ret = $this->memcache->add($_key, time(), self::DEF_LOCK_EXPIRE);
            $wait_timeout--;
            if ($ret || $wait_timeout < 0) {
                break;
            }
            sleep(1);
        }
        if ($this->debug) {
            error_log(sprintf('%s("%s,") -> %s', __METHOD__, $_key, $ret));
        }
        return $ret;
    }
}
